import path from 'path';
import * as R from 'ramda';
import fs from 'fs-extra';
import { MarkdownTransformer } from './markdown/MarkdownTransformer.js';
import prettier from 'prettier';
import { rimraf } from 'rimraf';
import { Ajv } from 'ajv';
import { Type } from '@sinclair/typebox';
import { Frontmatter } from '../../lib/schemas/Frontmatter.schema.js';
import Bluebird from 'bluebird';
import { createHash } from 'node:crypto';

class PipelineBoxBuilder {
    log;
    box;
    srcdir;
    outdir;
    transformer;
    ajv;
    constructor(log, box, srcdir, outdir) {
        this.log = log;
        this.box = box;
        this.srcdir = srcdir;
        this.outdir = outdir;
        this.transformer = new MarkdownTransformer(log, path.resolve(srcdir), path.resolve(outdir));
    }
    async build(files) {
        await rimraf(this.outdir);
        // convert all files to tasks
        const errors = await Bluebird.mapSeries(files, async (filename) => {
            const filelog = this.log.child({ name: filename });
            try {
                await this.bundle_file(filelog, filename);
                this.log.info(`✅ ${filename}`);
                return true;
            }
            catch (e) {
                this.log.error(`❌ [build] ${filename} - ${e.message}`);
                return false;
            }
        });
        const total = R.sum(errors.map((e) => (e ? 0 : 1)));
        if (total > 0) {
            this.log.error(`${total} files with errors`);
            throw new Error("Cannot process all files");
        }
        this.log.info(`All files processed!`);
    }
    async bundle_file(log, filename) {
        const f = path.join(this.srcdir, filename);
        const parsed = path.parse(f);
        const bundle_name = parsed.name;
        const cwd = path.resolve(parsed.dir);
        const source = (await fs.readFile(f)).toString();
        // Compile the file
        const { code, frontmatter } = await this.transformer.bundle_markdown({
            bundle_name,
            cwd,
            source,
        });
        const ajv = new Ajv({ allErrors: true });
        ajv.addSchema(Type.Strict(Frontmatter));
        const valid = ajv.validate("Frontmatter", frontmatter);
        if (!valid) {
            ajv.errors.forEach((error) => {
                return log.error(`❌ [markdown${error.instancePath}] ${error.message}`);
            });
            throw new Error(`Error building file ${filename}`);
        }
        const post = {
            data: code,
            title: frontmatter.title,
            title_short: frontmatter.title_short,
            subtitle: frontmatter.subtitle,
            modeldata: frontmatter.modeldata,
            lang: frontmatter.lang,
            categories: frontmatter.categories,
            social_cover: frontmatter.social_cover,
            authors: frontmatter.authors,
        };
        const extraction = {
            md5: createHash("md5").update(source).digest("hex"),
            post,
        };
        // Save post inside bundle directory
        const formatted_post = prettier.format(JSON.stringify(extraction), {
            semi: false,
            parser: "json",
        });
        await fs.ensureDir(this.outdir);
        const bundle_out_file = path.join(this.outdir, parsed.name + ".json");
        await fs.writeFile(bundle_out_file, formatted_post);
    }
}

export { PipelineBoxBuilder };
